#!/usr/bin/env node

var
redis = require("redis"),
client = redis.createClient(),
regenboog = require("./regenboog");

var algorithms = [];
for (var algorithm in regenboog.algorithms) {
  algorithms.push(algorithm);
}

var argv = require("optimist").string("password").string(algorithms).argv;

function usage() {
  console.log("cli.js [options]");
  console.log("--password\t<password>");
  console.log("Or one of the following:");

  for (var i in algorithms) {
    var algorithm = algorithms[i];
    console.log("--" + algorithm + "\t\t<hash>");
  }

  console.log("--help\tUsage");
}

exports.usage = usage;

function main() {
  var
  mode = "usage",
  algorithm = "",
  hash = "",
  hashes = {};

  if ("password" in argv) {
    mode = "encrypt";
    var password = argv.password;

    hashes = regenboog.encrypt(client, password);

    for (algorithm in hashes) {
      hash = hashes[algorithm];
      console.log(algorithm.toUpperCase() + ":\t" + hash);
    }
  }
  else {
    for (var i in algorithms) {
      algorithm = algorithms[i];

      if (algorithm in argv) {
        mode = "decrypt";
        hash = argv[algorithm];
        hashes[algorithm] = hash;
        break;
      }
    }

    if (mode === "usage") {
      usage();
    }
    else if (mode === "decrypt") {
      regenboog.decrypt(client, hashes, function (err, data) {
        if (data) {
          console.log("Decrypted: " + data);
        }
        else {
          console.log("No match.");
        }
      });
    }
  }

  client.quit();
}

if (!module.parent) { main(); }
