var
  smbhash = require("smbhash"),
  mhash = require("mhash");

var algorithms = {
  lm: smbhash.lmhash,
  nt: smbhash.nthash,
  md2: function (p) { return mhash.hash("md2", p); },
  md4: function (p) { return mhash.hash("md4", p); },
  md5: function (p) { return mhash.hash("md5", p); },
  sha1: function (p) { return mhash.hash("sha1", p); },
  sha256: function (p) { return mhash.hash("sha256", p); },
  mysql4: function (p) { return mhash.hash("sha1", mhash.hash("sha1", p)); }
};

exports.algorithms = algorithms;

var nAlgorithms = 0;
for (var algorithm in algorithms) {
  nAlgorithms++;
}

exports.nAlgorithms = nAlgorithms;

// Rainbow table key.
function key(algorithm, hash) {
  return algorithm + ":" + hash;
}

// Generates, stores, and returns hashes for a given password.
// Requires a Redis client.
function encrypt(client, password) {
  if (!password) {
    return {};
  }

  var hashes = {};

  for (var algorithm in algorithms) {
    var hash = algorithms[algorithm](password).toUpperCase();
    hashes[algorithm] = hash;

    var k = key(algorithm, hash);

    client.setnx(k, password);
  }

  return hashes;
}

exports.encrypt = encrypt;

// Searches rainbow tables for matching hash.
//
// Callback is in the form function (err, data) { ... }
// where data is either a password string or undefined.
//
// Requires a Redis client.
function decrypt(client, hashes, callback) {
  for (var algorithm in hashes) {
    var hash = hashes[algorithm].toUpperCase();
    var k = key(algorithm, hash);

    client.get(k, callback);
  }
}

exports.decrypt = decrypt;
