#!/usr/bin/env node

var
http = require("http"),
path = require("path"),
url = require("url"),

paperboy = require("paperboy"),
webroot = path.join(path.dirname(__filename), "public"),

binfix = require("binfix"),

redis = require("redis"),
client = redis.createClient(),
regenboog = require("./regenboog");

function formatHTML(query, res) {
  var
  password = query.password,
  hashes = {},
  hash = "",
  mode = "encrypt",
  status = "",
  algorithm = "",
  myurl = "";

  if (password) {
    myurl += "?password=" + password;
  }

  // Hash input -> mode "decrypt".
  for (algorithm in regenboog.algorithms) {
    hash = query[algorithm];

    if (hash) {
      hashes[algorithm] = hash;
      mode = "decrypt";
      myurl += "?" + algorithm + "=" + hash;
    }
    else {
      hashes[algorithm] = "";
    }
  }

  if (mode === "encrypt") {
    // Defaults to "password".
    password = password || "password";
    hashes = regenboog.encrypt(client, password);
    status = "Encrypted";
  }
  else {
    password = "";
    status = "Decrypting";
  }

  var html = "<!DOCTYPE html>" +
    "<html xmlns=\"http://www.w3.org/1999/xhtml\" lang=\"en\" xml:lang=\"en\">" +
    "<head>" +
    "<title>REGENBOOG</title>" +
    "<link rel=\"icon\" href=\"favicon.ico\" />" +
    "<link rel=\"stylesheet\" href=\"stylesheet.css\" type=\"text/css\" />" +
    "</head>" +
    "<body>" +
    "<nav>" +
    "<h1><a href=\"/\">REGENBOOG</a></h1>" +
    "<h3 id=\"pronunciation\">reh &middot; hen &middot; booh</h3>" +
    "</nav>" +

  "<article>";

  html += "<form action=\"\" method=\"get\">" +
    "<p><label for=\"password\">PASSWORD:</label> <input id=\"password\" type=\"text\" name=\"password\" value=\"" + password + "\" /> <span id=\"password-hover\" class=\"enter\" style=\"visibility: hidden;\">Click + Enter to Search</span></p>" +
    "</form>";

  for (algorithm in regenboog.algorithms) {
    html += "<form action=\"\" method=\"get\">" +
      "<p>" +
      "<label for=\"" + algorithm + "\">" + algorithm.toUpperCase() + ":</label>" +
      "<input id=\"" + algorithm + "\" type=\"text\" name=\"" + algorithm + "\" value=\"" + hashes[algorithm] + "\" /> " +
      "<span id=\"" + algorithm + "-hover\" class=\"enter\" style=\"visibility: hidden;\">Click + Enter to Search</span>" +
      "</p>" +
      "</form>";
  }

  html += "<p>" +
    "<label for\"status\">STATUS:</label>" +
    "<input id=\"status\" type=\"text\" name=\"status\" value=\"" + status + "\" readonly=\"readonly\" />" +
    "</p>";

  html += "<footer class=\"left\"><p>Other formats: ";

  var fmts = [];

  for (var format in formatters) {
    if (format !== "html") {
      fmts.push("<a href=\"/regenboog." + format + myurl + "\">" + format.toUpperCase() + "</a>");
    }
  }

  html += fmts.join(" | ");

  html += "</p></footer>";

  html += "<footer class=\"right\"><p><a href=\"https://www.paypal.com/cgi-bin/webscr?cmd=_donations&business=donate%40yellosoft%2eus&item_name=YelloSoft%20Team&no_shipping=0&no_note=1&tax=0&currency_code=USD&lc=US&bn=PP%2dDonationsBF&charset=UTF%2d8\">Donate</a> | <a href=\"about.html\">About</a></p>";

  html += "<p>So long, and thanks for <span id=\"total\">all the</span> passwords!</p></fooger>";

  html += "<script src=\"jquery-1.7.1.min.js\" type=\"text/javascript\"></script>" +
    "<script src=\"rainbow.min.js\" type=\"text/javascript\"></script>" +
    "<script src=\"ajax.js\" type=\"text/javascript\"></script>" +
    "<script src=\"analytics.min.js\" type=\"text/javascript\"></script>";

  html += "<script type=\"text/javascript\">";

  html += "$(\"#" + "password" +  "\").hover(" +
    "function () {" +
    "$(\"#" + "password" +  "-hover\").css(\"visibility\", \"visible\");" +
    "}," +
    "function () {" +
    "$(\"#" + "password" +  "-hover\").css(\"visibility\", \"hidden\");" +
    "});";

  for (algorithm in regenboog.algorithms) {
    html += "$(\"#" + algorithm +  "\").hover(" +
      "function () {" +
      "$(\"#" + algorithm +  "-hover\").css(\"visibility\", \"visible\");" +
      "}," +
      "function () {" +
      "$(\"#" + algorithm +  "-hover\").css(\"visibility\", \"hidden\");" +
      "});";
  }

  html += "</script>";

  if (mode === "decrypt") {
    for (algorithm in hashes) {
      hash = hashes[algorithm];

      if (hash !== "") {
        html += "<script type=\"text/javascript\">" +
          "decrypt(\"" + algorithm + "\", \"" + hash + "\");" +
          "</script>";
      }
    }
  }

  html += "</body>" + "</html>";

  res.end(html);
}

function modeHashesPassword(query) {
  var
  mode = "encrypt",
  hashes = {},
  password = query.password || "password";

  for (var algorithm in regenboog.algorithms) {
    var hash = query[algorithm];

    if (hash) {
      mode = "decrypt";
      hashes[algorithm] = hash;
    }
  }

  return { mode: mode, hashes: hashes, password: password };
}

function formatJSON(query, res) {
  var
  mhp = modeHashesPassword(query);
  mode = mhp.mode,
  hashes = mhp.hashes,
  password = mhp.password,
  total = query.total;

  if (total) {
    mode = "total";
    client.dbsize(function (err, total) {
      res.end(JSON.stringify({ total: binfix.format(total / regenboog.nAlgorithms) }));
    });
  }

  // No hash inputs.
  if (mode === "encrypt") {
    res.end(
      JSON.stringify({
        status: "Encrypted",
        password: password,
        hashes: regenboog.encrypt(client, password)
      })
    );
  }
  else if (mode === "decrypt") {
    regenboog.decrypt(client, hashes, function (err, data) {
      var o = { status: "NoMatch", hashes: hashes };

      if (data) {
        password = data;
        o = { password: password, status: "Decrypted" };
        o.hashes = regenboog.encrypt(client, password);
      }

      res.end(JSON.stringify(o));
    });
  }
}

function formatTXT(query, res) {
  var
  mhp = modeHashesPassword(query);
  mode = mhp.mode,
  hashes = mhp.hashes,
  password = mhp.password,
  total = query.total;

  if (total) {
    mode = "total";
    client.dbsize(function (err, total) {
      res.end("Total: " + binfix.format(total / regenboog.nAlgorithms));
    });
  }

  // No hash inputs.
  if (mode === "encrypt") {
    var txt = "Password: " + password + "\n";
    hashes = regenboog.encrypt(client, password);

    for (var algorithm in hashes) {
      var hash = hashes[algorithm];
      txt += algorithm.toUpperCase() + ": " + hash + "\n";
    }

    res.end(txt);
  }
  else if (mode === "decrypt") {
    regenboog.decrypt(client, hashes, function (err, data) {
      var txt = "No Match.\n";

      if (data) {
        password = data;
        txt = "Password: " + password + "\n";
        hashes = regenboog.encrypt(client, password);

        for (var algorithm in hashes) {
          var hash = hashes[algorithm];
          txt += algorithm.toUpperCase() + ": " + hash + "\n";
        }
      }

      res.end(txt);
    });
  }
}

var formatters = {
  "json": formatJSON,
  "txt": formatTXT,
  "html": formatHTML
};

var mimetypes = {
  "json": "text/json",
  "txt": "text/plain",
  "html": "text/html"
};

function server() {
  http.createServer(function (req, res) {
    paperboy.deliver(webroot, req, res).otherwise(function (error) {
      var
      query = url.parse(req.url, true).query,
      format = url.parse(req.url, true).pathname || "/";

      if (format.indexOf(".") !== -1) {
        format = format.split(".");
      }

      if (format.length > 0 && format[0] !== "") {
        format = format[format.length -1];
      }

      if (! (format in formatters)) {
        format = "html";
      }

      res.writeHead(200, {"Content-Type": mimetypes[format]});
      formatters[format](query, res);
    });
  }).listen(8765);

  console.log("Server started at http://localhost:8765/");
}

server();
